local function CheckAndCallSecure(functionName, ...)
	if IsProtectedFunction(functionName) then
		CallSecureProtected(functionName, ...)
	else
		_G[functionName](...)
	end
end

local function HarvensStackSplitSlider_Initialize(eventType, addonName)
	if addonName ~= "HarvensStackSplitSlider" then return end
	
	local control = WINDOW_MANAGER:CreateControlFromVirtual("HarvensStackSplitSlider", ZO_StackSplit, "ZO_Slider")
	control:SetAnchor(TOPLEFT, ZO_StackSplitSource, BOTTOMLEFT, 0, 16)
	control:SetAnchor(TOPRIGHT, ZO_StackSplitDestination1, BOTTOMRIGHT, 0, 16)
	control:SetHandler("OnMouseWheel", function(self, delta)
		local val = ZO_StackSplit.spinner.value
		val = val + delta
		if val > ZO_StackSplit.spinner.max then
			val = ZO_StackSplit.spinner.max
		end
		if val < ZO_StackSplit.spinner.min then
			val = ZO_StackSplit.spinner.min
		end
		
		local h = control:GetHandler("OnValueChanged")
		h(control, val)
	end)
	
	local checkbox = WINDOW_MANAGER:CreateControlFromVirtual("HarvensStackSplitCheckbox", ZO_StackSplit, "ZO_CheckButton")
	checkbox:SetAnchor(TOPLEFT, control, BOTTOMLEFT, 0, 16)
	ZO_CheckButton_SetLabelText(checkbox, "Multisplit")
	
--[[
	local editControl = WINDOW_MANAGER:CreateControlFromVirtual("HarvensStackSplitEdit", ZO_StackSplit, "ZO_DefaultEdit")
	local _, point,relative,relPoint, x, y = ZO_StackSplit.spinner.display:GetAnchor(0)
	editControl:SetAnchor(point,relative,relPoint, x+8, y)
	editControl:SetColor(1,1,1,1)
	editControl:SetMaxInputChars(2)
	editControl:SetTextType(TEXT_TYPE_NUMERIC_UNSIGNED_INT)
	editControl:SetWidth(24)
	editControl:SetHandler("OnFocusLost", function(self) 
		local val = tonumber(self:GetText())
		if val > ZO_StackSplit.spinner.max then
			val = ZO_StackSplit.spinner.max
		end
		if val < ZO_StackSplit.spinner.min then
			val = ZO_StackSplit.spinner.min
		end
		self:SetText(val)
		control:GetHandler("OnValueChanged")(control,val)
	end)
	ZO_StackSplit.spinner.display = editControl
--]]

	ZO_StackSplit.spinner:RegisterCallback("OnValueChanged", function(spinner)
		control:SetValue(ZO_StackSplit.spinner.value)
	end)
	local h = ZO_StackSplit:GetHandler("OnEffectivelyShown")
	ZO_StackSplit:SetHandler("OnEffectivelyShown", function(...)
		h(...)
		control:SetMinMax(ZO_StackSplit.spinner.min, ZO_StackSplit.spinner.max)
		control:SetValue(ZO_StackSplit.spinner.value)
		control:SetValueStep(ZO_StackSplit.spinner.step)
		control:SetHandler("OnValueChanged", function(control, value)
			if value < ZO_StackSplit.spinner.max then
				ZO_StackSplit.spinner.increaseButton:SetEnabled(true)
			else
				ZO_StackSplit.spinner.increaseButton:SetEnabled(false)
			end
			if value > ZO_StackSplit.spinner.min then
				ZO_StackSplit.spinner.decreaseButton:SetEnabled(true)
			else
				ZO_StackSplit.spinner.decreaseButton:SetEnabled(false)
			end
			ZO_StackSplit.spinner.value = value
			ZO_StackSplit.spinner.display:SetText(value)
			ZO_StackSplit.spinner:FireCallbacks("OnValueChanged", value)
		end)
	end)
	
	local dialog = ESO_Dialogs["SPLIT_STACK"]
	dialog.buttons[1].callback = function(control)
		local bag, index = ZO_Inventory_GetBagAndIndex(control.slotControl)
		local it = nil
		local emptySlotIndex
		local numStacks = 1
		if ZO_CheckButton_IsChecked(checkbox) then
			numStacks = math.floor(control.stackSize/control.spinner:GetValue())
			freeSlots = {}
			for i=0, GetBagSize(bag)-1 do
				if GetItemTotalCount(bag, i) == 0 then
					table.insert(freeSlots, i)
				end
			end
		else
			emptySlotIndex = FindFirstEmptySlotInBag(bag)
		end
		for i=1, numStacks do
			CheckAndCallSecure("PickupInventoryItem", bag, index, control.spinner:GetValue())
			if ZO_CheckButton_IsChecked(checkbox) then
				it, emptySlotIndex = next(freeSlots, it)
			end
			if emptySlotIndex ~= nil then
				CheckAndCallSecure("PlaceInInventory", bag, emptySlotIndex)
			else
				local errorStringId = (targetBag == BAG_BACKPACK) and SI_INVENTORY_ERROR_INVENTORY_FULL or SI_INVENTORY_ERROR_BANK_FULL
				ZO_Alert(UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, errorStringId)
				ClearCursor()
				break
			end
			ClearCursor()
		end
	end
end

EVENT_MANAGER:RegisterForEvent("HarvensStackSplitSliderInitialize", EVENT_ADD_ON_LOADED, HarvensStackSplitSlider_Initialize)